
/**********************************************************************

Copyright (C) 1995, 1996  E. O. Lawrence Berkeley National Laboratory.  
All Rights Reserved.  Permission to copy and modify this software and 
its documentation (if any) is hereby granted, provided that this notice
is retained thereon and on all copies.  

This software is provided as a professional academic contribution for 
joint exchange.   Thus it is experimental and scientific in nature, 
undergoing development, and is provided "as is" with no warranties of 
any kind whatsoever, no support, promise of updates or printed 
documentation.

This work is supported by the U. S. Department of Energy under contract 
number DE-AC03-76SF00098 between the U. S. Department of Energy and the 
University of California.


	Author: Wes Bethel
		Lawrence Berkeley Laboratory
                Berkeley, California 

  "this software is 100% hand-crafted by a human being in the USA"

***********************************************************************/


#include <stdio.h>
#include <string.h>

#include <avs/avs.h>
#include <avs/field.h>

#define DIVISION_IPU
#include "vr_util.h"
#include "matrix.h"

#define DEGREES_TO_RADIANS(a) ((a)*3.14159/180.0)
#define MAX(a,b) ((a) > (b) ? (a) : (b))

static AVSfield *head=NULL,*hand=NULL;

int
DivIPU_Desc()
{
    int p;
    
    AVSset_module_name("division IPU",MODULE_DATA);
    AVScreate_output_port("head",VR_FIELD_STRING); 
    AVScreate_output_port("hand",VR_FIELD_STRING); 
    
    p = AVSadd_parameter("dummy1","string","Module Sleep Control","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p=AVSadd_parameter("SleepToggle","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    

    p = AVSadd_parameter("dummy2","string","Rotation Control","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p = AVSadd_parameter("X Rotate","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Y Rotate","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Z Rotate","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_float_parameter("Rotation Scale",1.0,0.001,2.);

    p = AVSadd_parameter("dummy3","string","Translation Control","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p = AVSadd_parameter("X displacement","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Y displacement","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Z displacement","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_float_parameter("Translation Scale",0.75, 0.001, 10.);

    p = AVSadd_parameter("Dominant Attitude Filter","boolean",0,0,1);
    AVSconnect_widget(p,"toggle");
    AVSadd_parameter_prop(p,"width","integer",4);
    
    p = AVSadd_parameter("Dominant Translation Filter","boolean",0,0,1);
    AVSconnect_widget(p,"toggle");
    AVSadd_parameter_prop(p,"width","integer",4);
    
    p = AVSadd_parameter("Rotation Accumulation Mode","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    AVSadd_parameter_prop(p,"width","integer",4);
    
}

static char *d1,*d2,*d3;
static int xrot,yrot,zrot;
static int xtr,ytr,ztr;
float rot_scale,tr_scale;
float *rsptr= &rot_scale,*tsptr= &tr_scale;
static int sleep_toggle;
static int dominant_attitude_filter_enabled = 0;
static int dominant_translation_filter_enabled = 0;
static int accumulate_rotations = 1;
#define COROUTINE_INPUT() (AVScorout_input(&d1,&sleep_toggle,&d2,&xrot,&yrot,&zrot,&rsptr,&d3,&xtr,&ytr,&ztr,&tsptr,&dominant_attitude_filter_enabled,&dominant_translation_filter_enabled,&accumulate_rotations))
static matrix4x4 mlast[4],mthis[4],mthis_trans[4];
static matrix4x4 mlast_inverse[4],mthis_inverse[4];
static matrix4x4 macc[4],macc_inverse[4];

static float delta_azimuth[4],delta_elevation[4],delta_roll[4];

#define BUTTON_GOING_DOWN 1
#define BUTTON_DOWN 2
#define BUTTON_UP 4
#define BUTTUN_GOING_UP 8
static int button_state = BUTTON_UP;


main(ac,av)
int ac;
char *av[];
{
    AVSfield template;
    int nstations;
    int i,j,status;
    int just_woke_up;
    station_info sa[4];
    int buttons[5];

    vector3 xold,xnew,yold,ynew,zold,znew;
    double t1,t2;
    
    /* define the avs network editor UI for the spaceball module */
    AVScorout_init(ac,av,DivIPU_Desc);

    /* disable various parms until we write the code for them.... */

    AVSparameter_visible("Rotation Accumulation Mode",0);
    
    /* set up output fields.  first, do the head: */
    memset((char *)&template,0,sizeof(AVSfield));
    MAKE_VR_AVS_FIELD_TEMPLATE(&template);
    head = (AVSfield *)AVSfield_alloc(&template,VR_field_dims);

    memset((char *)(head->field_union.field_data_float_u),0,
	   sizeof(float)*VR_field_dims[0]); /* clear all memory */

    /* then, do the hand. */
    memset((char *)&template,0,sizeof(AVSfield));
    MAKE_VR_AVS_FIELD_TEMPLATE(&template);
    hand = (AVSfield *)AVSfield_alloc(&template,VR_field_dims);

    memset((char *)(hand->field_union.field_data_float_u),0,
	   sizeof(float)*VR_field_dims[0]); /* clear all memory */
    

    COROUTINE_INPUT();
    tr_scale = *tsptr;
    rot_scale = *rsptr;

    if (open_fastrak() == 0)
    {
	AVSfatal("Failed to open the fastrak device.  Bye.");
	exit(-1);
    }
    if (open_3dmouse() == 0)
    {
	AVSfatal("failed to open the 3d mouse device. bye. ");
	exit(-1);
    }

    /* enter a brief loop to try to reset the fastrak. this takes about
     10 secs or so clock time.  */

    if (reset_fastrak() == 0)
    {
	AVSfatal("Failed to reset the fastrak device.  Bye.");
	exit(-1);
    }

    /* configure the device */
    memset((char *)sa,0,sizeof(station_info)*MAX_STATIONS);
    nstations = configure_fastrak_stations(sa);

    /* reset the alignment frames for each of the various stations */
    status = realign_stations(sa,nstations);

    /* default behavior is to enable hardware filtering.  perhaps
     we should make this a parm on the module's UI */
    filter_enable();

    /* grab a few events to let things stabilize */
    for (i=0;i<12;i++)
    {
	status = get_fastrak_data(sa);
	normalize_rotations(sa[i].rot);
	build_composite_matrix(mlast+i,sa[i].rot);
    }


    for (i=0;;i++)			/* loop forever */
    {
	if (sleep_toggle)
	{
	    while(sleep_toggle)
	    {
		AVScorout_wait();
		COROUTINE_INPUT();
	    }
	    tr_scale = *tsptr;
	    rot_scale = *rsptr;
	    just_woke_up = 1;
	}


	/* do the buttons on the handheld mouse */
	memset((char *)buttons,0,sizeof(int)*5);
	status = get_mouse_data(buttons);
	set_button_flags(hand,buttons,status,nstations);
	

	/* read the tracker info */
	status = get_fastrak_data(sa);
	AVScorout_exec();
	
	if (status)
	{
	    for (j=0;j<nstations;j++)
	    {
		normalize_rotations(sa[j].rot);
		
		scale_events(sa[j].rot,&rot_scale);
		scale_events(sa[j].delta_pos,&tr_scale);

		build_composite_matrix(mthis+j,sa[j].rot);
		
		recompute_base_rotations(mthis+j,mlast+j,delta_azimuth+j,
					 delta_elevation+j,delta_roll+j);

	    }

	    j = 0;
	    do_rotation_events(hand,sa+j,delta_azimuth+j,delta_elevation+j,
			       delta_roll+j,&mouse3d,xrot,yrot,zrot,
			       dominant_attitude_filter_enabled);


 	    do_translation_events(hand,sa+j,mthis+j,&mouse3d,xtr,ytr,ztr,
				  dominant_translation_filter_enabled);

	    if (nstations > 1)
	    {
		j = 1;
		do_rotation_events(head,sa+j,delta_azimuth+j,delta_elevation+j,
				   delta_roll+j,&vr4,xrot,yrot,zrot,
				   dominant_attitude_filter_enabled);


		do_translation_events(head,sa+j,mthis+j,&vr4,xtr,ytr,ztr,
				      dominant_translation_filter_enabled);
	    }
	    memcpy((char *)mlast,(char *)mthis,sizeof(matrix4x4)*4);
	    
	    AVScorout_output(head,hand);
	}
	
	COROUTINE_INPUT();
	tr_scale = *tsptr;
	rot_scale = *rsptr;
    }
}


int
set_button_flags(hand,buttons,status,nstations)
int *buttons,status;
AVSfield *hand;
{
    int j;
    
    VR_SET_BUTTON(VR_BUTTON1_MASK,0,hand);
    VR_SET_BUTTON(VR_BUTTON2_MASK,0,hand);
    VR_SET_BUTTON(VR_BUTTON3_MASK,0,hand);
    VR_SET_BUTTON(VR_BUTTON4_MASK,0,hand);
    VR_SET_BUTTON(VR_BUTTON5_MASK,0,hand);
	
    if (status)
    {
	SET_BUTTONS_FLAG(1,hand); /* set flag to indicate a button event */
	if (buttons[0])
	    VR_SET_BUTTON(VR_BUTTON1_MASK,1,hand);

	if (buttons[1])
	    VR_SET_BUTTON(VR_BUTTON2_MASK,1,hand);

	if (buttons[2])
	    VR_SET_BUTTON(VR_BUTTON3_MASK,1,hand);

	if (buttons[3])
	{
	    /* button 4 is used by the vr_obj_ui module to indicate that
	       the user has "grabbed" the object, and it will be subsequently
	       transformed.  we care about the button state here to build
	       accumulated rotations (if set on this module's UI) */

	    /* 17 April 1996 - wes
	     note that the following code doesn't really do anything.
	     it is a throwback to previous incarnations, but hasn't
	     been ripped out yet. */

	    if (button_state == BUTTON_UP)
	    {
		button_state = BUTTON_GOING_DOWN;

		for (j=0;j<nstations;j++)
		{
		    identity_4x4(macc+j);
		    identity_4x4(macc_inverse+j);
		}
	    }
	    else if (button_state == BUTTON_GOING_DOWN)
		button_state = BUTTON_DOWN;
	    else if (button_state == BUTTON_DOWN)
		button_state = BUTTON_DOWN;
				
	    VR_SET_BUTTON(VR_BUTTON4_MASK,1,hand);
	}

	if (buttons[4])
	    VR_SET_BUTTON(VR_BUTTON5_MASK,1,hand);

    }
    else
    {
	SET_BUTTONS_FLAG(0,hand); /* set flag to indicate NO button event */
	/* clear all the individual button flags. */
	button_state = BUTTON_UP;
    }
}

int
do_rotation_events(f,sa,delta_az,delta_el,delta_roll,ft,xrot,yrot,zrot,
		   dominant_attitude_filter_enabled)
AVSfield *f;
station_info *sa;
float *delta_az,*delta_el,*delta_roll;
receiver_orientation *ft;
int xrot,yrot,zrot;
{
    float user_rot[3];
    extern double fabs();
    int i;

    /* azimuth corresponds to rotations about the receiver's z axis.
     change this to user space. */
    user_rot[ft->axes[2]] = *delta_az * ft->rot_scale_sign[2];

    /* elevation corresponds to rotations about the receiver's y axis */
    user_rot[ft->axes[1]] = *delta_el * ft->rot_scale_sign[1];;

    /* roll corresponds to rotations about the receiver's x axis */
    user_rot[ft->axes[0]] = *delta_roll * ft->rot_scale_sign[0];

    filter_events(user_rot,xrot,yrot,zrot);

    if (dominant_attitude_filter_enabled)
    {
	/* zero out all events except the maximum */
	
	int index;
	index = (fabs((double)user_rot[0]) > fabs((double)user_rot[1])) ? 0 : 1;
	index = (fabs((double)user_rot[index]) > fabs((double)user_rot[2])) ? index : 2;
	for (i=0;i<3;i++)
        {
	    if (i != index)
		user_rot[i] = 0.;
	}
    }
    
    VR_SET_ROT_VECTOR(user_rot[0],user_rot[1],user_rot[2],f);
}

int
do_translation_events(f,sa,m,ft,xtr,ytr,ztr,
		      dominant_translation_filter_enabled)
AVSfield *f;
station_info *sa;
matrix4x4 *m;
receiver_orientation *ft;
int xtr,ytr,ztr;
int dominant_translation_filter_enabled;
{
    vector4 tv1,tv2;
    matrix4x4 t;
    float user_trans[3];
    int i;
    extern double fabs();

    tv1.v[0] = sa->delta_pos[0];
    tv1.v[1] = sa->delta_pos[1];
    tv1.v[2] = sa->delta_pos[2];
    tv1.v[3] = 0.;		/* no homogeneous coords */

    transpose_4x4(m,&t);
    vector_matrix_mult_4(&tv1,m,&tv2);  

    for (i=0;i<3;i++)
	user_trans[i] = tv2.v[ft->translation_reassign[i]] * ft->trans_sign[i];

    filter_events(user_trans,xtr,ytr,ztr);

    if (dominant_translation_filter_enabled)
    {
	/* zero out all events except the maximum */
	
	int index;
	index = (fabs((double)user_trans[0]) > fabs((double)user_trans[1])) ? 0 : 1;
	index = (fabs((double)user_trans[index]) > fabs((double)user_trans[2])) ? index : 2;
	for (i=0;i<3;i++)
        {
	    if (i != index)
		user_trans[i] = 0.;
	}
    }
    VR_SET_TRANS_VECTOR(user_trans[0],user_trans[1],user_trans[2],f);
}


int
scale_events(raw_events,scale)
float *raw_events;
float *scale;
{
    raw_events[0] = raw_events[0] * *scale;
    raw_events[1] = raw_events[1] * *scale;
    raw_events[2] = raw_events[2] * *scale;
}


int
filter_events(raw_events,bool1,bool2,bool3)
float *raw_events;
int bool1,bool2,bool3;

{
    raw_events[0] = (bool1) ? (raw_events[0]) : 0.;
    raw_events[1] = (bool2) ? (raw_events[1]) : 0.;
    raw_events[2] = (bool3) ? (raw_events[2]) : 0.;
}

int
normalize_rotations(rot)
float *rot;
{
    int i;
    for (i=0;i<3;i++)
	rot[i] = (rot[i] < 0.) ? rot[i]+360. : rot[i];
}


int
matrix_to_direction_cosines(int index,
			    receiver_orientation *r,
			    matrix4x4 *m)
{
#if 0
    int i,j;

    for (j=0;j<3;j++)
    {
	r->xaxis[j] = m[index].m[0][j];
	r->yaxis[j] = m[index].m[1][j];
	r->zaxis[j] = m[index].m[2][j];
    }
#endif
}
