
/*********************************************************************

Copyright (C) 1994, Lawrence Berkeley Laboratory.  All Rights
Reserved.  Permission to copy and modify this software and its
documentation (if any) is hereby granted, provided that this notice
is retained thereon and on all copies.  

This software is provided as a professional academic contribution
for joint exchange.   Thus it is experimental and scientific
in nature, undergoing development, and is provided "as is" with
no warranties of any kind whatsoever, no support, promise of
updates or printed documentation.

This work is supported by the U. S. Department of Energy under 
contract number DE-AC03-76SF00098 between the U. S. Department 
of Energy and the University of California.


	Author: Wes Bethel
		Lawrence Berkeley Laboratory

  "this software is 100% hand-crafted by a human being in the USA"

*********************************************************************/


#include <stdio.h>
#include <string.h>

#include <avs/avs.h>
#include <avs/field.h>

#define SPACEBALL
#include "vr_util.h"
#undef SPACEBALL


static char default_spaceball_device[] = {"/dev/spaceball"};
static float rot_vec[3],trans_vec[3];
static float rscale,tscale;
int button_mask;


int
FloatDesc()
{
    int p;
    
    AVSset_module_name("spaceball",MODULE_DATA);
    AVScreate_output_port("output xform",VR_FIELD_STRING); 
    
    p = AVSadd_parameter("dummy1","string","Module Sleep Control","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p=AVSadd_parameter("SleepToggle","boolean",0,0,1);
    AVSconnect_widget(p,"toggle");

    p = AVSadd_parameter("dummy2","string","Rotation Control","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p = AVSadd_parameter("Z onoff","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Y onoff","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("X onoff","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_float_parameter("Rotation Scale",1.,0.001,10.);

    p = AVSadd_parameter("dummy3","string","Translation Control","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p = AVSadd_parameter("X displacement","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Y displacement","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_parameter("Z displacement","boolean",1,0,1);
    AVSconnect_widget(p,"toggle");
    p = AVSadd_float_parameter("Translation Scale",1., 0.001, 10000.);
    
    p = AVSadd_parameter("dummy4","string","Spaceball Device","","");
    AVSconnect_widget(p,"text");
    AVSadd_parameter_prop(p,"width","integer",4);
    p = AVSadd_parameter("spaceball_dev","string",default_spaceball_device,"","");
    AVSadd_parameter_prop(p,"width","integer",2);
}

int button_proc(),rotate_proc(),trans_proc();

static AVSfield *outfield=NULL;

static int zon,yon,xon;
static int xdisp,ydisp,zdisp;


main(argc,argv)
int argc;
char *argv[];
{
    
    static char *dummy1,*dummy2,*dummy3,*dummy4;
    float *rot_scale,*trans_scale;
    char *spaceball_tty;
    int sleep_toggle;
    int restart=0;
    int steps;
    int status;
    AVSfield template;
    int dims[1];

    /* define the avs network editor UI for the spaceball module */
    AVScorout_init(argc,argv,FloatDesc);

    /* the following two lines make heavy use of macros and
       structures defined in sb_util.h */
    memset((char *)&template,0,sizeof(AVSfield));
    MAKE_VR_AVS_FIELD_TEMPLATE(&template);
    outfield = (AVSfield *)AVSfield_alloc(&template,VR_field_dims);

    memset((char *)(outfield->field_union.field_data_float_u),0,
	   sizeof(float)*VR_field_dims[0]); /* clear all memory */
    
    /* wait for a spaceball tty if one isn't specified */
    spaceball_tty = NULL;
    AVScorout_input(&dummy1,&sleep_toggle,&dummy2,&zon,&yon,&xon,&rot_scale,&dummy3,&xdisp,&ydisp,&zdisp,&trans_scale,&dummy4,&spaceball_tty);

    if (spaceball_tty == NULL)
    {
	/* bark at user, spin until we get a spaceball_tty dev. */
	while(spaceball_tty == NULL)
	{
	    AVSwarning(" An invalid tty is being specified for the spaceball device.\n Try something like /dev/spaceball.");
	    
	    AVScorout_wait();
	    AVScorout_input(&dummy1,&sleep_toggle,&dummy2,&zon,&yon,&xon,&rot_scale,&dummy3,&xdisp,&ydisp,&zdisp,&trans_scale,&dummy4,&spaceball_tty);
	    
	}
    }

    rscale = *rot_scale;
    tscale = *trans_scale;
    
    /* have a string in spaceball_tty.  try to open the device.
       in the event of failure, an exit(1) call is buried deep in the
       spaceball library */

    status = open_named_spaceball(button_proc,rotate_proc,trans_proc,spaceball_tty);

    /* assume sb is now open.  the main loop consists of waiting for
       spaceball events or UI changes.  only when spaceball data is
       generated is a new field dumped downstream.  when the UI changes,
       it will take effect on the next spaceball data-driven event. */


    for (;;)
    {

	/* here, we want to grab spaceball events, and stuff them into
	   the output field. */

	status = WaitForBallData();
	while (status)
	{
	    SbWaitForPacket(1000);
	    SbCheckSpaceball();
	    SbRequestBallData();

	    /* stuff data to output port. */

	    AVScorout_output(outfield);
	    
	    AVScorout_exec();  /* wait for flow exec to stop running */
	    
	    SET_BUTTONS_FLAG(0,outfield);
	    SET_TRANSLATIONS_FLAG(0,outfield);
	    SET_ROTATIONS_FLAG(0,outfield);
	    
	    status = WaitForBallData();

	    /* reset flags in field to clear all events */

	}

	/* no ball data?  see if AVS has anything to say, but don't
	 stop here waiting for user input */
	
	AVScorout_input(&dummy1,&sleep_toggle,&dummy2,&zon,&yon,&xon,&rot_scale,&dummy3,&xdisp,&ydisp,&zdisp,&trans_scale,&dummy4,&spaceball_tty);
	rscale = *rot_scale;
	tscale = *trans_scale;

	/* spin here until sleep toggle is turned off  */
	while (sleep_toggle)
	{
	    AVScorout_wait();
	    AVScorout_input(&dummy1,&sleep_toggle,&dummy2,&zon,&yon,&xon,&rot_scale,&dummy3,&xdisp,&ydisp,&zdisp,&trans_scale,&dummy4,&spaceball_tty);
	}

    }
}

int
button_proc(button_vector)
SbUint16 button_vector;
{
    /** this code supports 8 buttons **/

    unsigned int bv;

    bv = button_vector;		/* this is for typing convenience */

    /* clear all button events */
    VR_SET_BUTTON(VR_BUTTON1_MASK,0,outfield);
    VR_SET_BUTTON(VR_BUTTON2_MASK,0,outfield);
    VR_SET_BUTTON(VR_BUTTON3_MASK,0,outfield);
    VR_SET_BUTTON(VR_BUTTON4_MASK,0,outfield);
    
    VR_SET_BUTTON(VR_BUTTON5_MASK,0,outfield);
    VR_SET_BUTTON(VR_BUTTON6_MASK,0,outfield);
    VR_SET_BUTTON(VR_BUTTON7_MASK,0,outfield);
    VR_SET_BUTTON(VR_BUTTON8_MASK,0,outfield);

    if (bv & 0x01ff)		/* any buttons pressed? */
    {
	SET_BUTTONS_FLAG(1,outfield); /* set flag to indicate
					 SOME button events */
	if (bv & 1)
	{
	    VR_SET_BUTTON(VR_BUTTON1_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 1 was pressed.\n"); */
	}
	if (bv & 2)
	{
	    VR_SET_BUTTON(VR_BUTTON2_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 2 was pressed.\n"); */
	}
	if (bv & 4)
	{
	    VR_SET_BUTTON(VR_BUTTON3_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 3 was pressed.\n"); */
	}
	if (bv & 8)
	{
	    VR_SET_BUTTON(VR_BUTTON4_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 4 was pressed.\n"); */
	}
	if (bv & 0x10)
	{
	    VR_SET_BUTTON(VR_BUTTON5_MASK,1,outfield);
/*	fprintf(stderr,"\tButton 5 was pressed.\n"); */
	}
	if (bv & 0x20)
	{
	    VR_SET_BUTTON(VR_BUTTON6_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 6 was pressed.\n"); */
	}
	if (bv & 0x40)
	{
	    VR_SET_BUTTON(VR_BUTTON7_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 7 was pressed.\n"); */
	}
	if (bv & 0x80)
	{
	    VR_SET_BUTTON(VR_BUTTON8_MASK,1,outfield);
/*	    fprintf(stderr,"\tButton 8 was pressed.\n"); */
	}
    }
    else
    {
	SET_BUTTONS_FLAG(0,outfield); /* set flag to indicate NO button event */
    }
    

}

int
rotate_proc(period,vec)
SbUint16 period;
SbReal vec[3];
{
    SET_ROTATIONS_FLAG(1,outfield);
    if (xon)
	rot_vec[0] = vec[0]*rscale;
    else
	rot_vec[0] = 0.;

    if (yon)
	rot_vec[1] = vec[1]*rscale;
    else
	rot_vec[1] = 0.;

    if (zon)
	rot_vec[2] = vec[2]*rscale;
    else
	rot_vec[2] = 0.;
    
    VR_SET_ROT_VECTOR(rot_vec[0],rot_vec[1],rot_vec[2],outfield);
}

int
trans_proc(period,vec)
SbUint16 period;
SbReal vec[3];
{
    int i;
    
/*    fprintf(stderr,"trans proc: %g %g %g\n",vec[0],vec[1],vec[2]);   */
    SET_TRANSLATIONS_FLAG(1,outfield);
    if (xdisp)
	trans_vec[0] = vec[0] * tscale;
    else
	trans_vec[0] = 0.;

    if (ydisp)
	trans_vec[1] = vec[1] * tscale;
    else
	trans_vec[1] = 0;

    if (zdisp)
	trans_vec[2] = vec[2] * tscale;
    else
	trans_vec[2] = 0.;

    VR_SET_TRANS_VECTOR(trans_vec[0],trans_vec[1],trans_vec[2],outfield);
}

#if 0
int
button_proc(keys)
SbUint16 keys;
{
    float sb[7];
    int i;

    for (i=0;i<7;i++)
	sb[i] = 0.;
    
    SbBeep("cE");
    if (keys & 0x1)  /* button 1 on the ball pressed */
    {
	/* do the reset */
	reset_all(&eye,&at,&up,sb,focal_length);
    }
    if (keys & 0x2) /* button 2 pressed on the ball */
    {
	if (dominant_mode == 1)
	{
	    AVSmodify_parameter("Dominant Toggle",AVS_VALUE,0,0,1);
/*	    fprintf(stderr,"\tdominant mode is ON.  turning it OFF.\n"); */
	    dominant_mode = 0;
	}
	else if (dominant_mode == 0)
	{
	    AVSmodify_parameter("Dominant Toggle",AVS_VALUE,1,0,1);
/*	    fprintf(stderr,"\tdominant mode is OFF.  turning it ON.\n"); */
	    dominant_mode = 1;
	}
    }
    if (keys & 0x100)  /* pick button pressed */
    {
	SbRezero();  /* rezero the ball */
    }
}
#endif
