
/*********************************************************************

Copyright (C) 1995, Lawrence Berkeley Laboratory.  All Rights
Reserved.  Permission to copy and modify this software and its
documentation (if any) is hereby granted, provided that this notice
is retained thereon and on all copies.  

This software is provided as a professional academic contribution
for joint exchange.   Thus it is experimental and scientific
in nature, undergoing development, and is provided "as is" with
no warranties of any kind whatsoever, no support, promise of
updates or printed documentation.

This work is supported by the U. S. Department of Energy under 
contract number DE-AC03-76SF00098 between the U. S. Department 
of Energy and the University of California.


	Author: Wes Bethel
		Lawrence Berkeley Laboratory

  "this software is 100% hand-crafted by a human being in the USA"

*********************************************************************/

#include <stdio.h>
#include <avs/avs.h>
#include <avs/field.h>
#include "vr_util.h"

AVSinit_modules()
{
    int Vr_2acc_float();
    AVSmodule_from_desc(Vr_2acc_float);
}

int
Vr_2acc_float()
{
    int Vr_2acc_float_compute();
    int p;
    
    AVSset_module_name("VR to acc floats",MODULE_FILTER);

    p=AVSadd_parameter("Reset All","boolean",0,0,1);
    AVSconnect_widget(p,"toggle");
    
    AVScreate_input_port("input field",VR_FIELD_STRING,REQUIRED);
    AVScreate_output_port("X Translation","real");
    AVScreate_output_port("Y Translation","real");
    AVScreate_output_port("Z Translation","real");
    
    AVScreate_output_port("X Rotation","real");
    AVScreate_output_port("Y Rotation","real");
    AVScreate_output_port("Z Rotation","real");

    AVSset_compute_proc(Vr_2acc_float_compute);
}

static float trans[3]={0.,0.,0.};
static float rot[3]={0.,0.,0.};

int
Vr_2acc_float_compute(inf,tx,ty,tz,rx,ry,rz,reset_flag)
AVSfield *inf;
float **tx,**ty,**tz;
float **rx,**ry,**rz;
int reset_flag;
{
    float *t;
    float dv_tr[3],dv_rot[3];
    int i;

    if (reset_flag)
    {
	for (i=0;i<3;i++)
        {
	    trans[i] = 0.;
	    rot[i] = 0.;
	}
	AVSmodify_parameter("Reset All",AVS_VALUE,0,0,0);
    }
    else
    {
	t = inf->field_union.field_data_float_u;
    
	VR_GET_TRANS_VECTOR(dv_tr[0],dv_tr[1],dv_tr[2],inf);
	
	for (i=0;i<3;i++)
	    trans[i] += dv_tr[i];
    
	VR_GET_ROT_VECTOR(dv_rot[0],dv_rot[1],dv_rot[2],inf);

	for (i=0;i<3;i++)
	    rot[i] += dv_rot[i];
    }
	
    *tx = trans;
    *ty = trans+1;
    *tz = trans+2;

    *rx = rot;
    *ry = rot+1;
    *rz = rot+2;

    return(1);
}
